/*
 * MPC9700.c
 *
 *  Created on: Feb 16, 2015
 *      Author: Mike Pappas
 */
#include "msp430.h"
#include "Sensors.h"
#include "RF430.h"
#include <math.h>       /* trunc */

extern unsigned char NDEF_Application_Data[];

unsigned int ADC_value=0, ADC_value2=0;
float temp=0, oldtemp=0, light=0;

unsigned char hex_ascii[10] = {0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39};

unsigned char message[5] = {0x0, 0x0, 0x0, 0x0, 0x0};

// Function to handle ADC and sensor measurements
void measureSensors() {
    //__delay_cycles(5000);
    ADC12CTL0 |= ADC12ENC | ADC12SC;         // Sampling and conversion start
    __bis_SR_register(LPM0_bits + GIE);      // LPM0, ADC12_ISR will force exit
    light = calculate_Light(ADC_value2);	 // Convert voltage measurements for light sensor
    temp = calculate_Temp(ADC_value);		 // Convert voltage measurements for temperature
    floatToAsciiTemp(temp);					 // Convert temperature to ASCII and add to NDEF message
    floatToAsciiLight(light);				 // Convert light measurement to ASCII and add to NDEF message
    __no_operation();
}

// ADC Vector function adapted from http://www.ti.com/lit/zip/slac536
// using MSP430FR59xx_adc12_01.c
// Written by: T. Witt / P. Thanigai, Texas Instruments Inc., November 2013
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC12_VECTOR))) ADC12_ISR (void)
#else
#error Compiler not supported!
#endif
{
  switch (__even_in_range(ADC12IV, ADC12IV_ADC12RDYIFG))
  {
    case ADC12IV_ADC12IFG1:                 // Vector 14:  ADC12MEM1 Interrupt
        ADC_value = ADC12MEM0;
        ADC_value2 = ADC12MEM1;
      __bic_SR_register_on_exit(LPM0_bits | GIE); // Exit active CPU, clear interrupts
      break;
    default: break;
  }
}

//Calculate Temperature based on ADC_value input voltage
float calculate_Temp(unsigned int ADC_value) {

	// Retrieve voltage measurement, subtract 500mv (thermistor base @ 0C)
	// Divide by 10 since thermistor reports 10mV per degree C
	return (ADC_value * CONVERSION_VALUE - 500) / 10;

}

//Calculate Light from input voltage and return
float calculate_Light(unsigned int ADC_value){

    // Retrieve voltage measurement for Light sensor
    return (ADC_value*CONVERSION_VALUE);

}


// Convert float temperature data into hex ascii for NDEF message
void floatToAsciiTemp(float temp){
	int tens = (int) trunc(temp/10);
	int ones = (int) trunc(temp)%10;
	int frac1 = (int) trunc(temp*10)%10;
	int frac2 = (int) trunc(temp*100)%10;

	NDEF_Application_Data[35] = hex_ascii[tens];
	NDEF_Application_Data[36] = hex_ascii[ones];
	NDEF_Application_Data[37] = 0x2E;
	NDEF_Application_Data[38] = hex_ascii[frac1];
	NDEF_Application_Data[39] = hex_ascii[frac2];

	return;
}

// Convert Light data into hex ascii for NDEF message
void floatToAsciiLight(float light){
	int thousands = (int) trunc(light/1000);
	int hundreds = (int) trunc(light/100)%10;
	int tens = (int) trunc(light/10)%10;
	int ones = (int) trunc(light)%10;

	NDEF_Application_Data[41] = hex_ascii[thousands];
	NDEF_Application_Data[42] = hex_ascii[hundreds];
	NDEF_Application_Data[43] = hex_ascii[tens];
	NDEF_Application_Data[44] = hex_ascii[ones];

	return;
}


